function Example01a
% just the five-point star, test stability and consistency, regular grid 
% but now with a full system
close all
clear all
global RBFscale
global RBFtype
global RBFpar
global RBFfactor
% notation as in the paper
m=4
d=2
format long g
% this is for the polyharmonic 
% RBFtype='tp'
% RBFsign=(-1)^(1+m-d/2)
% RBFfactor=RBFsign/(2^(2*m-1)*pi^(d/2)*factorial(m-1)*factorial(m-d/2)) 
% RBFscale=1
% RBFpar=2*m-d  % paper: 2*m-d>4, RBFpar=2*m-d even >4
% this is for Sobolev space
RBFtype='ms' % 
RBFscale=1 %
RBFpar=3% =m-d/2, so m=RBFpar+1 in 2D for type = ms
m=RBFpar+1
RBFfactor=2^(1-m)/gamma(m) % as in theory, see Wendland's book
% loop[ over the h in the table
hlist=[0.5 0.25 0.125 0.0625]; %  0.03125];
for hk=1:length(hlist)
    h=hlist(hk);
    [xp, yp]=meshgrid(-1:h:1,-1:h:1); % regular grid
    pts=[xp(:) yp(:)];
    npts=length(pts(:,1));
    figure
    plot(pts(:,1),pts(:,2),'.');
    axis square
    % get index list ibnd of boundary points
    ibnd1=find((abs(pts(:,1))>=1-eps)); % 
    ibnd2=find((abs(pts(:,2))>=1-eps)); % 
    ibnd=union(ibnd1, ibnd2);
    nbnd=length(ibnd);
    hold on
    plot(pts(ibnd,1),pts(ibnd,2),'rx');
    % get list iint of interior points
    iint=setdiff((1:npts)',ibnd);
    plot(pts(iint,1),pts(iint,2),'go');
    nint=length(iint);
    % prepare the stiffness matrix 
    A=zeros(npts, npts);
    % loop over interior points for stencils
    for k=1:nint
        ik=iint(k); % index of interior point in pts
        ds=distsqh(pts,pts(ik,:)); % squared distances
        [ms is]=sort(ds);  % sort them
        nlist=is(2:5); % the four closest neighbors
        % insert the stencil 
        A(ik,ik)=-4/(h*h);
        for j=2:5
            A(ik,is(j))=1/(h*h);
        end
    end
    % now the boundary points, as identity submatrix
    A(ibnd,ibnd)=eye(nbnd);
    % invert stiffnes matrix
    pA=inv(A);
    % this is the full stability constant, square matrix case
    snorm=norm(pA,Inf);
    figure
    spy(sparse(A));
    figure
    mesh(pA);
    % now the estimate via condition of transpose
    cA=condest(A');
    nA=norm(A,Inf);
    cnA=cA/nA; % the condition estimate
    % now for consistency. Take 5-point star at origin
    locpts=[[0 0]; [0 1]; [1 0]; [-1 0]; [0 -1] ]*h;
    sc=-[4 -1 -1 -1 -1]'/(h*h);
    AXX=kermat(locpts,locpts)*RBFfactor;
    ALX=laplacekermat(locpts, [0 0])*RBFfactor;
    ALL=laplaceXYkermat([0 0],[0 0])*RBFfactor;
    % and evaluate quadratic form 
    QFa=ALL-2*ALX'*sc+sc'*AXX*sc;
    erna=sqrt(QFa);
    % and the final product, as in the error analysis  of the paper
    prod=snorm*erna;
    % [nint h snorm erna prod]
    % prepare the table in LaTeX format
    disp(sprintf('%d & %6.4f & %f & %g & %g & %g \\\\',npts, h, snorm, cnA, erna, prod))
end
return
