function Example02a
% just the five-point star, test stability and consistency, regular grid 
% full matrix, but HERE: adding the worst case
close all
clear all
global RBFscale
global RBFtype
global RBFpar
global RBFfactor
RBFtype='ms' % 
RBFscale=1 %
RBFpar=3% =m-d/2, so m=RBFpar+1 in 2D for type = ms
m=RBFpar+1
RBFfactor=2^(1-m)/gamma(m) % as in theory, see Wendland's book
% get all points
hlist=[0.5 0.25 0.125 0.0625]; % the h used in the table 
LLh=length(hlist);
for hk=1:LLh %  length(hlist)  % loop over them 
    h=hlist(hk);
    [xp, yp]=meshgrid(-1:h:1,-1:h:1); % regular points 
    pts=[xp(:) yp(:)];
    npts=length(pts(:,1));
    figure
    plot(pts(:,1),pts(:,2),'.');
    axis square
    % extract boundary points
    ibnd1=find((abs(pts(:,1))>=1-eps)); % 
    ibnd2=find((abs(pts(:,2))>=1-eps)); % 
    ibnd=union(ibnd1, ibnd2);
    nbnd=length(ibnd);
    hold on
    plot(pts(ibnd,1),pts(ibnd,2),'rx');
    % interior points
    iint=setdiff((1:npts)',ibnd);
    plot(pts(iint,1),pts(iint,2),'go');
    title('Points')
    nint=length(iint);
    % full stiffness matrix 
    A=zeros(npts, npts);
    % loop over interior points and form 5-point star at neighbors
    for k=1:nint
        ik=iint(k); % index of interior point in pts
        ds=distsqh(pts,pts(ik,:)); % get distances squared 
        [ms is]=sort(ds);  % sort them, ascending.
        nlist=is(2:5); % the four closest neighbors
        A(ik,ik)=-4/(h*h); % the star at the center 
        for j=2:5
            A(ik,is(j))=1/(h*h); % and at the neighbors
        end
    end
    A(ibnd,ibnd)=eye(nbnd);
    pA=-inv(A); % restrict to interior poinrs and invert
    snorm=norm(pA,Inf); % take sup-sup norm 
    figure
    spy(sparse(A)); 
    title('Sparsity')
    figure
    mesh(pA);
    title('Inverse matrix')
    cA=condest(A);
    % now evaluate error norm on star 
    locpts=[[0 0]; [0 1]; [1 0]; [-1 0]; [0 -1] ]*h;
    sc=-[4 -1 -1 -1 -1]'/(h*h);
    AXX=kermat(locpts,locpts)*RBFfactor;
    ALX=laplacekermat(locpts, [0 0])*RBFfactor;
    ALL=laplaceXYkermat([0 0],[0 0])*RBFfactor;
    % the quadratic form
    QFa=ALL-2*ALX'*sc+sc'*AXX*sc;
    % and the error
    erna=sqrt(QFa);
    % compare with optimal stencil: calculate it
    sb=AXX\ALX;
    QFb=ALL-2*ALX'*sb+sb'*AXX*sb;
    ernb=sqrt(QFb);
    % take product with stability constant
    prod=snorm*erna;
    % [nint h snorm erna prod]
    % and output for LaTeX
    disp(sprintf('%d & %6.4f & %f & %f & %f \\\\',npts, h, snorm, erna, prod))
end
% now for worst-case 
% now the stability part
rs=sum(abs(pA'))'; % row sums 
[mr cr]=max(rs); % the max thereof
% we have to calculate the worst case for the L_infty norm, which
% consists of a vector of signs arising in the row with the largest L_1
% norm.
vv=sign(pA(cr,:))'; % the worst case in the domain of A^(-1)
uS=pA*vv; % the range of this, it is the u_S of the paper
nus=norm(uS,Inf) % norm of u_S
nAus=norm(vv,Inf) % norm of v=Au_S
SC2=nus/nAus % see first line of the proof
uS=uS/nus; % normalize
% now left part of Figure 1
figure
surf(xp, yp, reshape(uS,size(xp)))
title('Worst case function for stability')
% The consistency vector is constant. We take it at the center
ptssq=sum((pts.^2)')';  % total sum of squares of points
[mic ic]=min(ptssq); % get index ic of most centered point 
v=(A(ic,:)*kermat(pts,pts)*RBFfactor)'; % this is tilde lambda acting on K
uj=laplacekermat(pts,pts(ic,:))*RBFfactor-v; % this is the u_j of the paper
% now the right-hand part of Figure 1
figure
surf(xp, yp, reshape(uj,size(xp)))
title('Worst case function for consistency')
% We need its norm in the native space 
qf=laplaceXYkermat(pts(ic,:),pts(ic,:))*RBFfactor;
fu=laplaceXYkermat(pts, pts(ic,:))*RBFfactor;
fv=(A(ic,:)*laplacekermat(pts,pts)*RBFfactor)';
qf=qf-2*fv(ic)+v'*A(:,ic)
nuj=sqrt(qf) % this is norm of u_j in S, see second display in proof
% We now calculate  the f of the paper 
f=fu-fv;
figure
surf(xp, yp, reshape(f,size(xp)))
title('RHS for worst case')
% now the g of the paper
g=uj(ibnd);
rhs=zeros(npts,1);
rhs(iint)=f(iint);
rhs(ibnd)=uj(ibnd);
utest=A\rhs-uj;
ustar=uj;
KA=2;
alpha=(KA-1)*norm(A*ustar-rhs,Inf) % the alpha in the paper 
utilde=ustar+alpha*snorm*uS;
beta=norm(A*utilde-rhs,Inf)
quot=beta/alpha
% now the numbers in the  paper, first absolute bound
normus=norm(ustar-utilde,Inf)
upperbound=(KA+1)*SC2*nuj*erna
lowerbound=(KA-1)*SC2*nuj*erna
% now relative bound
relnormus=normus/nuj
relupperbound=upperbound/nuj
rellowerbound=lowerbound/nuj
% this is Figure 2
figure
surf(xp, yp, reshape(utilde,size(xp)))
title('Solution for worst case')

return
