function Example05
% work on Chebyshev nodes 
close all
clear all
rand('state',0)
global RBFscale
global RBFtype
global RBFpar
global RBFfactor
global RBFsign
m=6 % smoothness 
d=2 % dimension
RBFtype='tp' % thin-plate due to d=2
RBFsign=(-1)^(1+m-d/2) 
RBFfactor=RBFsign/(2^(2*m-1)*pi^(d/2)*factorial(m-1)*factorial(m-d/2)) 
RBFscale=1
RBFpar=2*m-d  % 
q=1+m-d/2 % polynomial order 
tol=1.e-12 % tolerance for PHStencil routine
mode='greedy' % mode for PHStencil routine
% n_one=17 % 1D number of points, something like 2^n+1, e.g. 5, 9, 17, 33
nlist=[9 17 33 65];
for nn=1:length(nlist)
    n_one=nlist(nn);
    h=2/(n_one-1);
    hp=pi/(n_one-1);
    [tx, ty]=meshgrid(0:hp:pi,0:hp:pi);
    ptsg=[-cos(tx(:)) -cos(ty(:))];
    xg=-cos(tx);
    yg=-cos(ty);
    npts=length(ptsg(:,1)); % number of total points 
    % get boundary points
    ixbnd=find(abs(ptsg(:,1))>=1-eps);
    iybnd=find(abs(ptsg(:,2))>=1-eps);
    ibnd=union(ixbnd, iybnd); % indices of boundary points
    iint=setdiff(1:npts,ibnd)'; % indices of interior points
    nint=length(iint); % number of interior points
    nbnd=length(ibnd); % number of boundary points
    pts=ptsg; % these will be the final points
    % replace interior ones by noised ones
    noise=0.01;
    pts(iint,:)=ptsg(iint,:)+(1-2*rand(size(ptsg(iint,:))))*noise;
    % now plot them
    figure
    plot(... %ptsg(:,1),ptsg(:,2),'.',...
        pts(ibnd,1), pts(ibnd,2),'r.',...
        pts(iint,1), pts(iint,2),'b.')
    title('Node distribution')
    % this finishes point generation
    nneig=max(q,30);  % number of neighbors to be selected, must be at least q. 
    A=sparse(npts, nneig); % stiffness matrix 
    B=zeros(npts,1); % vector for consistency error
    for i=1:nint  % loop over all interior points
        ip=iint(i); % point index
        % now neighbor generation, very primitive, replace by kd tree for real
        % world cases
        ds=distsqh(pts, pts(ip,:)); % squared halved distances
        [sds ids]=sort(ds); % sort them
        ineig=ids(1:nneig); % take the closest ones
        % now calculate a polyharmonic stancil 
        [co, I, en, epol]=PHStencil(m, q, pts(ineig,:), nneig, pts(ip,:), tol, mode);
        A(ip,ineig(I))=co(I)'; % insert as row 
        B(ip)=en; % save consistency error
    end
    % now add identity on boundary
    A(ibnd,ibnd)=eye(nbnd);
    figure
    spy(A)
    title('Sparsity pattern of full stiffness matrix')
    figure
    surfc(xg, yg, reshape(B,size(xg)))
    title('Consistency error at nodes')
    cA=condest(A');
    nA=norm(A,Inf);
    scA=cA/nA;
    % if npts<1100
    %     sA=norm(pinv(full(A)),Inf);
    % end
    consmax=max(abs(B(:)));
    prod=consmax*scA;
    figure
    plot(sort(B))
    title('Distribution of consistency error')
    % partial problem
    Apart=A(iint, iint);
    figure
    spy(Apart)
    title('Sparsity pattern of partial stiffness matrix')
    cApart=condest(Apart');
    nApart=norm(Apart,Inf);
    scApart=cApart/nApart;
    % if npts<1100
    %     sApart=norm(pinv(full(Apart)),Inf);
    % end
    prodpart=consmax*scApart;
    disp(sprintf('%4d & %4d &%8.4f & %8.4f & %10.8f & %10.8f \\\\',...
        npts, nint, h, scApart, consmax, prodpart))
end