function IS=mycolumnfinder(A, tol)
% A is an underdetermined nxm matrix.
% We want to find the first |IS| columns 
% that make the rank maximal.
% It is not allowed to permute columns, just to ignore some.
% The output set IS contains indices of linearly independent columns.
% Application: Working on a matrix of polynomial values 
% on neighbors of a center point, at increasing distance,
% until a stable unisolvent set of points is found.
% Method: column-pivoted QR, but without swapping columns, just ignoring,
% and without returning Q or R.
% 
[m,n]=size(A);
% For a final safety check, we might keep the input matrix.
% Aold=A;
if m==1 % treat trivial case  m==1, returning first nontrivial element index
    it=find(abs(A(1,:))>tol);
    if length(it)>0
        IS=[it(1)];
    else
        IS=[];
    end
    return
end
s=sum(A.*A); % the column sums of squares, works bad for m=1
k=1; % we perform k <=m steps 
j=1; % and work currently on the column j>=k, up to j==n
IS=[];
while 1==1
    % testing sums of squares of the remainder matrix
%     if k<m
%         testsquares=norm(sum(A(k:end,j:end).^2)-s(j:end),Inf);
%     else
%         testsquares=norm(A(k:end,j:end).^2-s(j:end),Inf);
%     end
    % run through the next j until finding a nontrivial column
%     s(j)
%     [s(j) tol]
    if s(j)<=tol 
        j=j+1;
        if j>n
            break;
        end
        continue;
    end
    % now we can work  on the rest of the array,  from (k,j)
    IS=[IS j]; % add new j to list
    if k==m % no QR necessary for k==m  
        s(j:end)=s(j:end)-A(k,j:end).^2;
        break; 
    end
    % perform QR from column j and row k on
    a=A(k:end,j);
    na=norm(a,2);
    b=a;
    % avoid the crazy sign function in MATLAB: 
    % b(1)=a(1)+sign(a(1))*na  fails for a(1)==0
    if a(1)<0
        b(1)=a(1)-na;
    else
        b(1)=a(1)+na;
    end
    if norm(b,2)<tol
        disp(sprintf('Small column, norm = %g, tol = %g', norm(b,2), tol));
%         [a b]
%         [j s(j)]
        break;
    end;
    u=b/norm(b,2);
    v=2*u'*A(k:end,j:end);
    A(k:end,j:end)=A(k:end,j:end)-u*v;
    s(j:end)=s(j:end)-A(k,j:end).^2;
    % This ends the Householder transformation
    k=k+1; % k==m is treated above, so no check for k
    j=j+1;
    if j>n
        break;
    end
end
% IS
% For rank testing, keep the old A matrix...
% testrank=rank(Aold(:,IS))-length(IS)