function testPHStencil01
% tests the general PHstencil routine on a single point set, but in all
% modes.
close all
clear all
global RBFscale
global RBFpar
global RBFtype
global RBFfactor
global RBFsign
% rand('state',0)  % uncomment for a single case
% input parameters 
tol=1.e-12  % the tolerance used 
m=4
d=2 % 
% get the minimal order of conditional positive definiteness
minorder=1+floor(m-d/2)  
% set the actual order >= minorder here 
q=minorder % actual order used 
% This determines  the polynomial dimension:
Q=factorial(q-1+d)/(factorial(q-1)*factorial(d)) % dimension of polynomial space
Qplus=factorial(q+d)/(factorial(q)*factorial(d)); % dimension of polynomial space
% for one order higher
% Now for definition of points. First the random case 
% Fix M and n first 
% M=2*Q
% % n=M
% n=min(M,Qplus) % a good option for random points
% % Completely random case in [-1,+1]^d
% points=1-2*rand(M,d);
% % but z not near the boundary
% z=0.5*(1-2*rand(1,d))
% one might add z to the random points
% points(1,:)=z;
% 
% Now the "regular" case, maybe with noise 
%
np=9 % 9, 17, 33 number of 1D points along boundary
hp=1
bnd=np-1;
[xp, yp]=meshgrid(0:hp:bnd,0:hp:bnd);
points=[xp(:) yp(:)];
noise=0.1
points=points+noise*rand(size(points));
M=length(points(:,1))
z=[bnd/2 bnd/2] % for interior case
% z=[0 0] % for corner case
% z=[bnd/2 0]  % for mid-boundary case. Careful: tends to have rank loss if n is too small
%
% we sort the points  by distance to z, close points first
% 
[ps, ips]=sort(sum(((points-repmat(z,M,1)).^2)'));
points=points(ips,:);
% For regular points, there may be severe rank loss.
% Thus we need a minimal n that works.
% We find a unisolvent set of neighbors
nmin=max(mycolumnfinder(polynomials(points,q)', tol))
n=min(M, nmin+(Qplus-Q))
% and we might reduce the M points to n points, to improve locality
points=points(1:n,:);
%
% From here on, there are no changes to be made for running the program
%
% m and d determine the harmonic kernel 
if mod(d,2)==0
    RBFtype='tp'
    RBFsign=(-1)^(1+m-d/2)
    RBFfactor=1/(2^(2*m-1)*pi^(d/2)*factorial(m-1)*factorial(m-d/2)) 
else
    RBFtype='p'
    RBFsign=(-1)^ceil(m-d/2)
    RBFfactor=gamma(m-d/2)/(2^(2*m)*pi^(d/2)*factorial(m-1))
end
RBFscale=1
RBFpar=2*m-d  % paper: 2*m-d>4, RBFpar=2*m-d even >4
if RBFpar <=4
    disp('Not enough regularity for the Laplacian, increase m or decrease d')
    return
end
if M<Q
    disp('Not enough points for the required polynomial exactness, increase M')
    return
end


[co_close, I_close, en_close, epol_close, n_close, L1n_close]=...
    SinglePHStencilRunWithPlot('close', m, q, points, n, z, tol);
[co_kernel, I_kernel, en_kernel, epol_kernel, n_kernel, L1n_kernel]=...
    SinglePHStencilRunWithPlot('kernel', m, q, points, n, z, tol);
[co_greedy, I_greedy, en_greedy, epol_greedy, n_greedy, L1n_greedy]=...
    SinglePHStencilRunWithPlot('greedy', m, q, points, n, z, tol);
[co_polL2, I_polL2, en_polL2, epol_polL2, n_polL2, L1n_polL2]=...
    SinglePHStencilRunWithPlot('polL2', m, q, points, n, z, tol);
[co_polL1, I_polL1, en_polL1, epol_polL1, n_polL1, L1n_polL1]=...
    SinglePHStencilRunWithPlot('polL1', m, q, points, n, z, tol);
[co_polback, I_polback, en_polback, epol_polback, n_polback, L1n_polback]=...
    SinglePHStencilRunWithPlot('polback', m, q, points, n, z, tol);


disp(sprintf('kernel   en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_kernel, epol_kernel, n_kernel, L1n_kernel))  
disp(sprintf('greedy   en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_greedy, epol_greedy, n_greedy, L1n_greedy))  
disp(sprintf('polL2    en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_polL2, epol_polL2, n_polL2, L1n_polL2))  
disp(sprintf('polL1    en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_polL1, epol_polL1, n_polL1, L1n_polL1))  
disp(sprintf('polback  en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_polback, epol_polback, n_polback, L1n_polback))  
disp(sprintf('close    en = %12.5e, epol = %12.5e, |I| = %3d L1norm = %12.5e',...
    en_close, epol_close, n_close, L1n_close))  

return
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [co, I, en, epol, n_fin, L1n]=...
    SinglePHStencilRunWithPlot(mode, m, q, points, n, z, tol)
% perforsm a single run and plots results.
% Returns n_fin als final length of stencil
% and L1n as final L1 norm of stencil.
[co, I, en, epol]=PHStencil(m, q, points, n, z, tol, mode);
figure
plot(points(:,1),points(:,2),'.',z(1),z(2),'rx',...
    points(I,1), points(I,2),'bo')
title(sprintf(...
    'Raw points and selected stencil points for %s method, norm =%g', mode, en))
n_fin=length(I);
L1n=sum(abs(co(I)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



